/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package home

import (
	"internal/hlp"

	"fmt"
	"os/user"
	"path"
	)

//-----------------------------------------------------------------------------
// A home directory that contain "list" files of "<string>, <string>" format

type Home struct {
	abs_path   string
	list_files []ListFile
	dumps      map[string]bool
	}

func (h *Home) absPath(name string) string {
	return path.Join(h.abs_path, name)
	}

type ListFile struct {
	name   string
	values map[string]string
	}

func NewListFile(name, content string) (*ListFile, error) {
	values := make(map[string]string)
	err := hlp.ParseCSV(content, values)
	if err != nil {
		return nil, err
		}
	p := ListFile{name: name, values: values}
	return &p, nil
	}

//-----------------------------------------------------------------------------
// Recreate the home dir and get the app configuration

func PrepareHome(app_name string, list_files []ListFile) (*Home, error) {
	usr, err := user.Current()
	if err != nil {
		return nil, err
		}
	h := new(Home)
	// Assigning the data fields
	h.abs_path = usr.HomeDir + "/." + app_name + "/"
	h.list_files = list_files
	// Creating a home dirrectory, creating and reading list files
	// and checking binary dumps
	steps := [] func() error {
		h.createDir,
		h.createLists,
		h.checkDumps,
		}
	for _, step := range steps {
		if err := step(); err != nil {
			return nil, err
			}
		}
	return h, nil
	}

//-----------------------------------------------------------------------------
// Get a value that has a given label in a list file with a given name

func (h *Home) GetVal(file_name string, label string) (string, error) {
	values := h.getValues(file_name)
	if values == nil {
		err := fmt.Errorf("Home directory was not properly prepared")
		return "", err
		}
	val, ok := values[label]
	if !ok {
		err := fmt.Errorf("There is no %q target in %q", label,
			h.absPath(file_name))
		return "", err
		}
	return val, nil
	}

//-----------------------------------------------------------------------------
// Form file path for a binary dump and check if the dump exists

func (h *Home) DumpPath(target string) (string, bool) {
	file_path := h.absPath(target + ".dmp")
	return file_path, h.dumps[target]
	}
