/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package main
import (
	"internal/ar"
	"internal/cfmt"
	"internal/hlp"
	"internal/strval"

	"fmt"
	"io/ioutil"
	"os"
	"path/filepath"
	"sort"
	"strings"
	"sync"
	"time"
	)

type worker struct {
	archive     *ar.Archive
	documents   chan document  // for passing task records of src packages
	semaphores  chan struct{}  // for maintaining a quota of goroutines
	parsing_wg  sync.WaitGroup // for waiting for parsers finish their jobs
	writing_wg  sync.WaitGroup // for waiting for a writer finish it's jobs
	}

type document struct {
	subject string
	entries []entry
	}

type entry  struct {
	task_id    ar.TaskId
	subtask_id ar.SubTaskId
	subtask    ar.SubTask
	}

func NewWorker(cpu_n int) *worker {
	var w worker
	w.archive    = ar.NewArchive()
	w.documents  = make(chan document, cpu_n)
	w.semaphores = make(chan struct{}, cpu_n)
	return &w
	}

//-----------------------------------------------------------------------------
// Handle an another file of a target dir

func (w *worker) handleFile(path string, f os.FileInfo, err error) error {

	// Checking if it's a searched file
	if !f.Mode().IsRegular() || filepath.Base(path) != "d-t-s-evr.list" {
		return nil
		}
	// Trying to read a file that is a searched file
	data, err := ioutil.ReadFile(path)
	if err != nil {
		return fmt.Errorf("Parsing %q, %v\n", path, err)
		}

	// Parsing this file in a new thread
	w.parsing_wg.Add(1)
	go w.parseFile(path, data)

	return nil
	}

//-----------------------------------------------------------------------------
// Parse a given data string of a "d-t-s-evr.list" file

func (w *worker) parseFile(path string, data []byte) {

	// Capturing a marker
	w.semaphores <- struct{}{}

	// Decrementing the counter of running threads and releasing the marker
	// on the way out
	defer func() {
		w.parsing_wg.Done()
		<- w.semaphores
		}()

	// Getting a name of a src package
	src_name := filepath.Base(filepath.Dir(path))

	// Preparing a container
	doc := document{src_name, make([]entry, 0, 10)}

	// For every line of data
	NextLine:
	for n, line := range strings.Split(string(data), "\n") {

		// Breaking up a line to a number of words
		columns := strings.Fields(line)
		if len(columns) < 4 || columns[2] == "-" {
			continue
			}

		// Getting values of all the fields from the columns

		var timestamp  = new(ar.Timestamp)
		var task_id    = new(strval.Unsigned)
		var subtask_id = new(strval.Unsigned)
		var ver_rel    = new(strval.Pair)
		var hash       = new(ar.HashSum)

		// Putting various data fields into one container
		var fields = make([]strval.Assignable, 0, 5)
		fields = append(fields, timestamp, task_id, subtask_id, ver_rel)
		if len(columns) > 4 {
			fields = append(fields, hash)
			}
		// Processing all fields stored in the container
		for i, p := range fields {
			if err := p.Assign(columns[i]); err != nil {
				hlp.PrintWarn(fmt.Sprintf("Format violation in file %q " +
					"on line %d: %s", path, n, err.Error()))
				continue NextLine
				}
			}
		// Forming a sub task description
		src_package := ar.SrcPackage{0, ver_rel.First, ver_rel.Second}
		subtask := ar.SubTask{*timestamp, src_package, *hash}
		doc.entries = append(doc.entries,
			entry{ar.TaskId(*task_id), ar.SubTaskId(*subtask_id), subtask})
		}

	w.documents <- doc
	}

//-----------------------------------------------------------------------------
// Writing a gathered data into the archive

func (w *worker) writeDocuments() {
	// Decrementing the counter of running threads on the way out
	defer w.writing_wg.Done()
	for doc := range w.documents {
		for _, e := range doc.entries {
			w.archive.Write(doc.subject, e.task_id, e.subtask_id, e.subtask)
			}
		}
	}

//-----------------------------------------------------------------------------
// Select tasks that match a given criteria

func (w *worker) getMatchingTasks(task_id uint, src_name string,
		after_date, before_date time.Time) ([]ar.TaskId, error) {

	var task_ids  []ar.TaskId
	var selection map[ar.TaskId]bool
	var err error
	got_range := !after_date.IsZero() || !before_date.IsZero()

	if task_id != 0 {
		// Finding task with a given id
		if ok := w.archive.FindTaskId(ar.TaskId(task_id)); ok {
			task_ids = []ar.TaskId{ar.TaskId(task_id)}
			}
		return task_ids, nil
		}
	if src_name != "" {
		// Finding tasks for a given src name and sorting them
		task_ids, err = w.archive.FindTasksBySrc(src_name, nil)
		// Forming a selection set to use with other conditions
		if got_range {
			selection = make(map[ar.TaskId]bool)
			if task_ids != nil {
				for _, task_id := range task_ids {
					selection[task_id] = true
					}
				}
		} else if task_ids != nil {
			// Sorting the slice of task ids
			sort.Slice(task_ids, func(i, j int) bool {
				return task_ids[i] < task_ids[j]
				})
			}
		}
	if got_range {
		// Finding tasks for a given time frame
		task_ids, err = w.archive.FindTasksByDate(after_date, before_date,
			selection)
		}

	return task_ids, err
	}

//-----------------------------------------------------------------------------
// Print info of a tasks with given ids

func (w *worker) showTasks(task_ids []ar.TaskId, format cfmt.CFormat) {
	// If no tasks have been found
	if task_ids == nil || len(task_ids) == 0 {
		fmt.Println("-- none --")
		return
		}
	// Printing all the matched tasks
	for _, task_id := range task_ids {
		w.archive.PrintTask(task_id, format)
		}
	}

//-----------------------------------------------------------------------------
// Select tasks that match a given criteria and print the selection

func (w *worker) showMatchingTasks(task_id uint, src_name string,
		after_date, before_date time.Time, format cfmt.CFormat, verbose bool) error {

	task_ids, err := w.getMatchingTasks(task_id, src_name, after_date, before_date)
	if err != nil {
		return err
		}

	if verbose {
		hlp.PrintDelimLine()
		defer hlp.PrintDelimLine()
		}

	w.showTasks(task_ids, format)
	return nil
	}

//-----------------------------------------------------------------------------
// Make/load a binary dump

const _NOT_INITIALIZED = "archive is not initialized"

func (w *worker) makeDump(path string) error {
	if w.archive == nil {
		return fmt.Errorf(_NOT_INITIALIZED)
		}
	// Encoding an archive
	b, err := w.archive.Encode()
	if err != nil {
		return err
		}
	// Writing the encoded archive into the file
	return ioutil.WriteFile(path, b, 0644)
	}

func (w *worker) loadDump(path string) error {
	if w.archive == nil {
		return fmt.Errorf(_NOT_INITIALIZED)
		}
	// Reading binary file
	b, err := ioutil.ReadFile(path)
	if err != nil {
		return err
		}
	// Decoding the slice of bytes of an encoded archive
	return w.archive.Decode(b)
	}
